# GazeApp
This is the code of CVPRW 2017 paper "It's Written All Over Your Face: Full-Face Appearance-Based Gaze Estimation". It provides the completed implementation from the input video/image to gaze direction/projection on the display for multiple users.

# Usage
## 1. Install OpenFace
We use the face and facial landmark detection from the OpenFace (https://github.com/TadasBaltrusaitis/OpenFace). You can also replace the face detection method by yourself.

## 2. Install Caffe
We use the Caffe (https://github.com/BVLC/caffe) for the CNN model. Please install the original Caffe first, and then put the files in "caffe-mod" to the Caffe folder, at last re-compile Caffe.

## 3. Camera & screen calibration information
Put you camera intrinsic parameters in the file "content/calib/calibration.yml".  
We use the OpenCV format for the camera calibration parameters.

(Optional) Put your camera-screen relationship parameter in the file "content/calibration/calib/monitor.yml", which is used to project the 3D gaze direction in the screen coordinate system.

## 4. Fix camera focal length (Optional)
For the camera live mode, if you want to fix the camera focal lenth, you have to install "uvcdynctrl" and "guvcview" as following:

```bash
sudo apt install uvcdynctrl
sudo apt install guvcview
```

## 5. Dowload Models
You can download the pre-trained Caffe model for face patch:  
https://drive.google.com/file/d/0B5Q6yhOwf8aaaW91V0phU19PaW8/view?usp=sharing  
Caffe model for eye image:  
https://drive.google.com/file/d/0B5Q6yhOwf8aaVkplQVFHdjBXY2c/view?usp=sharing  

The SVM eye contact detection model can be downloaded from here:  
https://drive.google.com/file/d/0B5Q6yhOwf8aaaEphOElybDJsQjA/view?usp=sharing  
Which has to be used together with the caffe model modified for eye contact detection:  
https://drive.google.com/file/d/0B5Q6yhOwf8aaZ3ZzVWV4bkdwRGc/view?usp=sharing

## 6. Compile our software
Rename "CMakeLists.txt.example" to "CMakeLists.txt", and modify paths for OpenCV, Boost, OpenMP, Caffe and OpenFace.

```bash
mkdir build
cd build
ccmake ..
make
```

## 7. Run the example
Rename "default.cfg.example" to "default.cfg" and change the paths according to your environment.
Now you can run the application as "./GazeApp -i ../example/example.avi -o ../example/ -d -v"

You can also specify your own config file via --config option.
For more details of program options, please refer to the source code.

# Model selection
The code use the full-face patch model specifically in the "default.cfg" file. It will load the model file "content/caffeModel/face_patch.caffemodel" that trained on both MPIIGaze and EYEDIAP HD video. Please refer to our paper [2] for more details.
If you want to use the single eye image gaze estimation [1], you should first specifically add "face_model=0" in the default.cfg, and also modify the "cnn_param_path" and "cnn_model_path" to the eye model.

# Citation
Please cite our papers if you use the code and/or pre-trained models.  
[1] Appearance-Based Gaze Estimation in the Wild. Xucong Zhang, Yusuke Sugano, Mario Fritz, Andreas Bulling. CVPR 2015.  
[2] It's Written All Over Your Face: Full-Face Appearance-Based Gaze Estimation. Xucong Zhang, Yusuke Sugano, Mario Fritz, Andreas Bulling. CVPRW 2017.

# Copyright
This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/4.0/ or send a letter to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.

# Final remarks
Bug report and suggestion are welcome. We would be grateful for your understanding that this is research code and not full fledged product and we are working on to make it robust and easy to use.
