#ifndef COMMON_H
#define COMMON_H

#include <opencv2/opencv.hpp>

namespace gaze_data
{
    struct faceinfo
    {
        unsigned long face_id;
        double certainty;
        cv::Point2f landmarks[6];
        cv::Rect_<double> face_bb;
    };

    struct eyeinfo
    {
        unsigned long face_id;
        cv::Mat head_r, head_t;
        cv::Vec3f leye_pos, reye_pos;

        // normalized eyes
        cv::Mat leye_img, reye_img;
        cv::Mat leye_rot, reye_rot;
        cv::Mat leye_conv, reye_conv;

        // gaze vectors
        cv::Vec2f lgaze_angle, rgaze_angle; // 2-dimension gaze direction in the normalized space
        cv::Vec3f lgaze_vec, rgaze_vec; // 3-dimension gaze direction vector in the camera coordinate system
    };

    struct facepatchinfo
    {
        unsigned long face_id;
        cv::Mat head_r, head_t;
        cv::Vec3f face_pos;

        // normalized eyes
        cv::Mat face_img;
        cv::Mat face_rot;
        cv::Mat face_conv;

        // gaze vectors
        cv::Vec2f gaze_angle; // 2-dimension gaze direction in the normalized space
        cv::Vec3f gaze_vec; // 3-dimension gaze direction vector in the camera coordinate system
        float feature[4096];
    };

    struct gazeinfo
    {
        unsigned long face_id;

        // projected gaze positions on the screen coordinate
        cv::Point2f lgaze, rgaze; // the gaze projection of left eye and right eye for the eye model case.
        cv::Point2f gaze; // the gaze projection on the screen for the face model case.
    };
}

#endif // COMMON_H
