#ifndef DISPLAYSPACE_H
#define DISPLAYSPACE_H

#include <vector>
#include <string>

#include <opencv2/opencv.hpp>

#include "common.h"

class DisplaySpace
{
public:
    DisplaySpace(std::string calib_file, std::string display_file);
    ~DisplaySpace();

    void project_to_display(std::vector<gaze_data::eyeinfo> &eyes,
                            std::vector<gaze_data::gazeinfo> &gazes, cv::Mat &debug_img);
    void project_to_display(std::vector<gaze_data::facepatchinfo> &facepatches,
                            std::vector<gaze_data::gazeinfo> &gazes, cv::Mat &debug_img);
    void project_to_display(std::vector<gaze_data::eyeinfo> &eyes,
                            std::vector<gaze_data::gazeinfo> &gazes)
    {
        cv::Mat dummy;
        project_to_display(eyes, gazes, dummy);
    }
    void project_to_display(std::vector<gaze_data::facepatchinfo> &facepatches,
                            std::vector<gaze_data::gazeinfo> &gazes)
    {
        cv::Mat dummy;
        project_to_display(facepatches, gazes, dummy);
    }

    void debug_show(cv::Mat &debug_img, std::vector<gaze_data::eyeinfo> &eyes);

private:
    cv::Mat camera_matrix_, dist_coeffs_;
    cv::Mat monitor_R_, monitor_T_;
    float monitor_W_, monitor_H_;
    cv::Vec3f monitor_corners_[4];
    cv::Vec3f monitor_normal_;

    void intersect_gaze(const cv::Vec3f origin, const cv::Vec3f gaze_vec, cv::Point2f &gaze_pos);
};

#endif // DISPLAYSPACE_H
