#ifndef FACETRACKER_H
#define FACETRACKER_H

// wrapper class for CLM face tracker
// ref. MultiTrackCLM.cpp

#include <vector>
#include <string>

#include "LandmarkCoreIncludes.h"
#include "common.h"

class FaceTracker
{
public:
    FaceTracker(std::string model_dir);
    ~FaceTracker();

    void track_faces(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &results, cv::Mat &debug_img);
    void track_faces(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &results)
    {
        cv::Mat dummy;
        track_faces(input_img, results, dummy);
    }

    void reset();

private:
    void NonOverlapingDetections(const std::vector<LandmarkDetector::CLNF>& clm_models,
                                 std::vector<cv::Rect_<double> >& face_detections);

    double NonOverlapingDetection(const LandmarkDetector::CLNF &ref_model, const LandmarkDetector::CLNF &tgt_model);

    vector<LandmarkDetector::FaceModelParameters> det_parameters_;
    vector<LandmarkDetector::CLNF> clnf_models_;
    vector<bool> active_models_;
    vector<unsigned long> face_ids_;

    unsigned long current_face_id_;

    int frame_counter_;

    static const int num_faces_max_, detection_skip_frames_, tracking_loss_limit_;
    static const float detection_resize_rate_;
    static const float nonoverlap_threshold_;
    static const double certainty_threshold_;
    static const int landmark_indices_[6];
};

#endif // FACETRACKER_H
