#ifndef GAZEESTIMATOR_H
#define GAZEESTIMATOR_H

#include <opencv2/opencv.hpp>

// caffe
#define USE_OPENCV 1;
#include <caffe/caffe.hpp>
#include <caffe/util/io.hpp>
#include <caffe/blob.hpp>
#include <caffe/layers/pose_data_layer.hpp>
#include <caffe/layers/memory_data_layer.hpp>

#include "common.h"

class GazeEstimator
{
public:
    GazeEstimator(std::string param_path, std::string model_path);
    ~GazeEstimator();

    void estimate(std::vector<gaze_data::eyeinfo> &eyes);
    void estimate(std::vector<gaze_data::facepatchinfo> &facepatches);

private:
    caffe::Net<float> *p_net_;

    void prepare_data(const cv::Mat &src_eye, const cv::Mat &src_rot,
                      cv::Mat &dst_eye, cv::Vec2f &dst_rot, const bool is_right);
    void prepare_data(const cv::Mat &src_face, const cv::Mat &src_rot,
                                 cv::Mat &dst_face, cv::Vec2f &dst_rot);

    void process_data(const cv::Mat &eye, const cv::Vec2f &rot, cv::Vec2f &gaze, int img_wdith, int img_height, float* feature=NULL);
    void extract_feature();
    void convert_gaze(const cv::Vec2f &gaze_angle, const cv::Mat &cnv_mat,
                      cv::Vec3f &gaze_vec, const bool is_right);

    int model_type_;
};

#endif // GAZEESTIMATOR_H
