#ifndef HEADPOSEESTIMATOR_H
#define HEADPOSEESTIMATOR_H

// estimate head poses via shape model fitting

#include <vector>
#include <string>

#include <opencv2/opencv.hpp>

#include "common.h"

class HeadPoseEstimator
{
public:
    HeadPoseEstimator(std::string calib_file, std::string model_file);
    ~HeadPoseEstimator();

    void obtain_eyepatches(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &landmarks,
                           std::vector<gaze_data::eyeinfo> &results, cv::Mat &debug_img);
    void obtain_eyepatches(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &landmarks,
                           std::vector<gaze_data::eyeinfo> &results)
    {
        cv::Mat dummy;
        obtain_eyepatches(input_img, landmarks, results, dummy);
    }
    void obtain_facepatches(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &landmarks,
                            std::vector<gaze_data::facepatchinfo> &results, cv::Mat &debug_img);
    void obtain_facepatches(cv::Mat &input_img, std::vector<gaze_data::faceinfo> &landmarks,
                            std::vector<gaze_data::facepatchinfo> &results)
    {
        cv::Mat dummy;
        obtain_facepatches(input_img, landmarks, results, dummy);
    }

    void draw_gaze(cv::Mat &debug_img, std::vector<gaze_data::eyeinfo> &eyes);
    void draw_gaze(cv::Mat &debug_img, std::vector<gaze_data::facepatchinfo> &facepatchinfo);

private:
    void read_cameradata(std::string data_path);
    void read_monitordata(std::string data_path);

    cv::Mat camera_matrix_, dist_coeffs_;
    cv::Mat face_model_mat_;
    std::vector<cv::Point3f> face_model_;

    void draw_landmarks(cv::Mat &img, const cv::Mat &rvec, const cv::Mat &tvec);
    void draw_headpose(cv::Mat &img, const cv::Mat &rvec, const cv::Mat &tvec);

    void get_normalized_data(const cv::Mat &img, gaze_data::eyeinfo &eye_info);
    void get_normalized_face(const cv::Mat &img, gaze_data::facepatchinfo &facepatch);
    void get_normalized_eye(const cv::Mat &img, const cv::Mat &head_R, const cv::Mat &eye_pos,
                            cv::Mat &eye_img, cv::Mat &eye_pose, cv::Mat &eye_conv);
    void get_normalized_face_patch(const cv::Mat &img, const cv::Mat &head_R, const cv::Mat &eye_pos,
                            cv::Mat &eye_img, cv::Mat &eye_pose, cv::Mat &eye_conv);
};

#endif // HEADPOSEESTIMATOR_H
